% Series expansion of the modified Bessel function of first kind
%
clear all;
epsilon=1e-9;
N=50; % Maximum order included in the calculation
yr=(pi/256:pi/256:10*pi);
y=[-fliplr(yr),0,yr];
%
% Calculation of the modified Bessel function of the first kind
%
for j=1:N+1,
    n=j-1;
    A=1i^n*(y/2).^n;
    c=1/factorial(n);
    k=0;
    Ik=c*ones(1,length(y));
    Incr=ones(1,length(y));
    while max(abs(Incr./Ik))>epsilon
        c=(k+1)/(k+1+n)*c;
        k=k+1;
        Incr=(-1)^k*c*(y.^k/(2^k*factorial(k))).^2;
        Ik=Ik+Incr;
    end;
    if sign((-1)^n)==1, % Even order
        Ie(j,:)=A.*Ik;
    else
        Io(j,:)=A.*Ik;
    end;
    I(j,:)=A.*Ik;
end;
%
% Graphics of modified Bessel function of the first kind
%
figure(1);
clf;
subplot(211);
plot(y/pi,real(Ie),y/pi,imag(Ie));
grid on;
xlabel('Argument j\theta/\pi');
ylabel('Amplitude');
title('Modified Bessel function of the first kind - Even order');
legend;
subplot(212);
plot(y/pi,real(Io),y/pi,imag(Io));
grid on;
xlabel('Argument j\theta/\pi');
ylabel('Amplitude');
title('Modified Bessel function of the first kind - Odd order');
%
% Calculation of the spectral coefficients for selected angular amplitude
%
Theta_points=[0,pi/16,pi/8,pi/4,pi/2,pi,2*pi,4*pi];
% Theta_points=[pi,2*pi,3*pi,4*pi,6*pi,8*pi,10*pi,12*pi];
for i=1:length(Theta_points),
    for j=1:N+1,
        n=j-1;
        W(i,j)=2*pi*(-1)^n*I(j,y==Theta_points(i));
    end;
    figure(2);
    subplot(4,2,i);
    hold on;
    bar(-(0:N),real(W(i,:)),0.2,'b');
    bar((0:N),real(W(i,:)),0.2,'b');
    bar(-(0:N),imag(W(i,:)),0.2,'r');
    bar((0:N),imag(W(i,:)),0.2,'r');
    hold off;
    grid on;
    xlabel('Normalized frequency \omega/\omega_s');
    ylabel('Amplitude');
    title(['Spectrum of the sinusoidal phase modulation - \theta_0=' num2str(Theta_points(i)/pi,'%3.3f') '\pi']);
    %
    figure(3);
    subplot(4,2,i);
    hold on;
    bar(-(0:N),abs(W(i,:)),0.2,'k');
    bar((0:N),abs(W(i,:)),0.2,'k');
    hold off;
    grid on;
    xlabel('Normalized frequency \omega/\omega_s');
    ylabel('Linear magnitude');
    title(['Spectrum of the sinusoidal phase modulation - \theta_0=' num2str(Theta_points(i)/pi,'%3.3f') '\pi']);
end;
